/*
 * SimilarityMeasure.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-02
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity;

/**
 * This package contains classes for calculation of various similarity and 
 * dissimilatity measures.
 *
 * Originally written as standalone program by Rick Lupia "SimCalc.c" (v 1.0),
 * changed to functions returning a matrix by M. Kosnik, 1998.11.24,
 * updated with Simpson 1998.12.10, m-kosnik@uchicago.edu;
 * functions returning similarities, dissimilarities, distances, associations 
 * & correlations added by P. Cejchan, cej@cejchan.gli.cas.cz, 1999/08/12.
 * PearsonCorrS_row revised 2002/02/24 by Jia Ke (kejia_2000@yahoo.com).
 * Copies of the original source code are available without cost from:
 * http://geosci.uchicago.edu/paleo/csource/
 * <p>
 * Converted to a Java package with a class for each measure by Martin Hassel, 
 * xmartin@nada.kth.se
 * <br>If anyone wishes to flesh out the "descriptions" of the indvidual measures, 
 * they are more than welcome :-)
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-02
 */
public abstract class SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public abstract double[][] analyzeMatrix(float[][] data_matrix);
}
