/*
 * BinPearsonCorrS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Oct-01
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * BINARY PEARSON CORRELATION COEFFICIENT 
 * 
 * <br>http://crrm.univ-mrs.fr/commercial/software/dataview/appendix.html
 * <br>similarity [-1, 1]
 * <br>Usage: for binary data.
 * 
 * @author  Martin Hassel
 * @version 2004-Oct-01
 */
public class BinPearsonCorrS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		int a = 0; /* present in both samples */
		int b = 0; /* present in sample i, absent in sample j */
		int c = 0; /* present in sample j, absent in sample i */
		int d = 0; /* absent in both samples */
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				a = b = c = d = 0;
				for(int column = 0; column < columns; ++column) {
					if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 1)
						a++;
					else if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 0)
						b++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 1)
						c++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 0)
						d++;
				}
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					((a*d)-(b*c))/Math.sqrt((a+b)*(a+c)*(b+d)*(c+d));
			}
		}
		return sim_matrix;
	}
}
