/*
 * MeanEuclidD_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Nov-06
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * MEAN EUCLIDEAN DISTANCE
 * 
 * @author  Martin Hassel
 * @version 2004-Nov-06
 */
public class MeanEuclidD_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double dist_matrix[][]; /* matrix to contain the distance */
		int rows = data_matrix.length;
		EuclidD_row distance = new EuclidD_row();

		dist_matrix = distance.analyzeMatrix(data_matrix);
		for(int row = 0; row < rows; ++row) {
			for(int column = (row + 1); column < rows; ++column) {
				dist_matrix[column][row] = dist_matrix[row][column] /= rows;
			}
		}
		return dist_matrix;		
	}
}
