/*
 * RobinsonS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-13
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * ROBINSON SIMILARITY COEFFICIENT - ("INDEX OF AGREEMENT")
 * 
 * <br>Marquardt, W.H. 1978 Archaeological seriation. In: Schiffer, M.B.(ed.) 
 * Advances in Archaeological Method and Theory. Academic Press, N.Y., p.264. 
 * Robinson, W.S. 1951 A method for chronologically ordering archaeological 
 * deposits. American Antiquity, 16: 293-301.
 * <p>
 * <b>WARNING: it rewrites data_martix with percentages!</B>
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-13
 */
public class RobinsonS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double sum;
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 200 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 200.0;
		
		/* calculate percentages */
		for(int row = 0; row < rows; ++row) {
			int rowsum = 0;
			for(int column = 0; column < columns; ++column)
				rowsum += data_matrix[row][column];
			for(int column = 0; column < columns; ++column)
				data_matrix[row][column] *= 100.0/rowsum;
		}
		
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = row1+1; row2 < rows; ++row2) {
				sum = 0;
				for(int column = 0; column < columns; ++column)
					sum += Math.abs(data_matrix[row1][column]-data_matrix[row2][column]);
				sim_matrix[row1][row2]= sim_matrix[row2][row1] = 200 - sum;
			}
		}
		return sim_matrix;		
	}
}
