/*
 * ScaledTaxonomicS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-13
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * SCALED TAXONOMIC SIMILARITY COEFFICIENT
 * 
 * <br>Marquardt, W.H. 1978 Archaeological seriation. In: Schiffer, M.B.(ed.) 
 * Advances in Archaeological Method and Theory. Academic Press, N.Y., p.290-291.
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-13
 */
public class ScaledTaxonomicS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double sim_matrix[][], max; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* call taxonomic distance */
		SimilarityMeasure sm = new TaxonomicD_row();
		sim_matrix = sm.analyzeMatrix(data_matrix);
		
		/* find maximum value */
		max = 0.0;
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < columns; ++row2) {
				if(max < sim_matrix[row1][row2])
					max = sim_matrix[row1][row2];
			}
		}
		
		/* scale */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < columns; ++row2)
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 1.0 - sim_matrix[row1][row2]/max;
		}
		
		/* Set diagonal to 1.0 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		return sim_matrix;		
	}
}
